@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002 Joaqun Cuenca Abela
+ * Copyright (C) 2002 Joaquín Cuenca Abela
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -17,7 +17,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors:
- *   Joaqun Cuenca Abela <e98cuenc@yahoo.com>
+ *   Joaquín Cuenca Abela <e98cuenc@yahoo.com>
  *   Archit Baweja <bighead@users.sourceforge.net>
  */
 #ifdef HAVE_CONFIG_H
@@ -43,12 +43,20 @@
 #include "glade-app.h"
 #include "glade-fixed-manager.h"
 
+
+/* Concerning placeholders: we do not hold any reference to placeholders,
+ * placeholders that are supplied by the backend are not reffed, placeholders
+ * that are created by glade-command are temporarily owned by glade-command
+ * untill they are added to a container; in which case it belongs to GTK+
+ * and the backend (but I prefer to think of it as the backend).
+ */
 typedef struct {
 	GladeWidget      *widget;
 	GladeWidget      *parent;
 	GladeProject     *project;
 	GladePlaceholder *placeholder;
 	GList            *pack_props;
+	gulong		 handler_id;
 } CommandData;
 
 static GObjectClass *parent_class = NULL;
@@ -367,7 +375,7 @@ glade_command_push_undo (GladeProject *project, GladeCommand *cmd)
 	else
 		project->prev_redo_item = g_list_next (project->prev_redo_item);
 
-	glade_default_app_update_ui ();
+	glade_app_update_ui ();
 }
 
 /**************************************************/
@@ -397,7 +405,7 @@ glade_command_set_property_undo (GladeCommand *cmd)
 	return glade_command_set_property_execute (cmd);
 }
 
-/**
+/*
  * Execute the set property command and revert it. IE, after the execution of 
  * this function cmd will point to the undo action
  */
@@ -499,7 +507,7 @@ glade_command_set_property_collapse (GladeCommand *this, GladeCommand *other)
 	this->description = other->description;
 	other->description = NULL;
 
-	glade_default_app_update_ui ();
+	glade_app_update_ui ();
 }
 
 
@@ -655,7 +663,7 @@ glade_command_set_name_undo (GladeCommand *cmd)
 	return glade_command_set_name_execute (cmd);
 }
 
-/**
+/*
  * Execute the set name command and revert it.  Ie, after the execution of this
  * function cmd will point to the undo action
  */
@@ -725,7 +733,7 @@ glade_command_set_name_collapse (GladeCommand *this, GladeCommand *other)
 	g_free (this->description);
 	this->description = g_strdup_printf (_("Renaming %s to %s"), nthis->name, nthis->old_name);
 
-	glade_default_app_update_ui ();
+	glade_app_update_ui ();
 }
 
 /* this function takes the ownership of name */
@@ -770,6 +778,7 @@ typedef struct {
 	GList            *widgets;
 	gboolean          create;
 	gboolean          from_clipboard;
+	gboolean          props_recorded;
 } GladeCommandCreateDelete;
 
 GLADE_MAKE_COMMAND (GladeCommandCreateDelete, glade_command_create_delete);
@@ -785,14 +794,39 @@ glade_command_create_delete_undo (GladeCommand *cmd)
 	return glade_command_create_delete_execute (cmd);
 }
 
+static void 
+glade_command_placeholder_destroyed (GtkObject *object, CommandData *cdata)
+{
+	if (GTK_OBJECT (cdata->placeholder) == object)
+	{
+		cdata->placeholder = NULL;
+		cdata->handler_id = 0;
+	}
+}
+
+static void
+glade_command_placeholder_connect (CommandData *cdata,
+				   GladePlaceholder *placeholder)
+{
+	g_assert (cdata && cdata->placeholder == NULL);
+
+	/* Something like a no-op with no placeholder */
+	if ((cdata->placeholder = placeholder) == NULL)
+		return;
+
+	cdata->handler_id = g_signal_connect 
+		(placeholder, "destroy",
+		 G_CALLBACK (glade_command_placeholder_destroyed), cdata);
+}
+
 static gboolean
 glade_command_create_execute (GladeCommandCreateDelete *me)
 {
-	GladeClipboard   *clipboard = glade_default_app_get_clipboard();
+	GladeClipboard   *clipboard = glade_app_get_clipboard();
 	CommandData      *cdata = NULL;
-	GList            *list, *wlist = NULL;
+	GList            *list, *wlist = NULL, *l;
 
-	glade_default_app_selection_clear (FALSE);
+	glade_app_selection_clear (FALSE);
 
 	for (list = me->widgets; list && list->data; list = list->next)
 	{
@@ -814,12 +848,51 @@ glade_command_create_execute (GladeCommandCreateDelete *me)
 			}
 			else
 			{
+				glade_widget_set_parent (cdata->widget, cdata->parent);
+
 				glade_widget_class_container_add 
 					(cdata->parent->widget_class,
 					 cdata->parent->object,
 					 cdata->widget->object);
-				glade_widget_set_parent (cdata->widget, cdata->parent);
+
+				glade_widget_set_packing_properties (cdata->widget, cdata->parent);
 			}
+
+			/* Now that we've added, apply any packing props if nescisary. */
+			for (l = cdata->pack_props; l; l = l->next)
+			{
+				GValue         value = { 0, };
+				GladeProperty *saved_prop = l->data;
+				GladeProperty *widget_prop = 
+					glade_widget_get_pack_property (cdata->widget,
+									saved_prop->class->id);
+				
+				glade_property_get_value (saved_prop, &value);
+				glade_property_set_value (widget_prop, &value);
+				g_value_unset (&value);
+			}
+				
+			if (me->props_recorded == FALSE) 
+			{
+				/* Save the packing properties after the initial creation.
+				 * (this will be the defaults returned by the container
+				 * implementation after initially adding them).
+				 *
+				 * Otherwise this recorded marker was set when deleting
+				 */
+				g_assert (cdata->pack_props == NULL);
+				for (l = cdata->widget->packing_properties; l; l = l->next)
+					cdata->pack_props = 
+						g_list_prepend (cdata->pack_props,
+								glade_property_dup (GLADE_PROPERTY (l->data),
+										    cdata->widget));
+
+
+				/* Mark the properties as recorded
+				 */
+				me->props_recorded = TRUE;
+			}
+
 		}
 		
 		if (me->from_clipboard)
@@ -831,7 +904,7 @@ glade_command_create_execute (GladeCommandCreateDelete *me)
 			glade_project_add_object 
 				(GLADE_PROJECT (cdata->widget->project),
 				 NULL, cdata->widget->object);
-			glade_default_app_selection_add 
+			glade_app_selection_add 
 				(cdata->widget->object, TRUE);
 		}
 	}
@@ -851,7 +924,7 @@ glade_command_create_execute (GladeCommandCreateDelete *me)
 static gboolean
 glade_command_delete_execute (GladeCommandCreateDelete *me)
 {
-	GladeClipboard   *clipboard = glade_default_app_get_clipboard();
+	GladeClipboard   *clipboard = glade_app_get_clipboard();
 	CommandData      *cdata;
 	GList            *list, *wlist = NULL;
 
@@ -862,11 +935,9 @@ glade_command_delete_execute (GladeCommandCreateDelete *me)
 		if (cdata->parent)
 		{
 			if (cdata->placeholder)
-			{
 				glade_widget_replace
 					(cdata->parent, cdata->widget->object, 
 					 G_OBJECT (cdata->placeholder));
-			}
 			else if (cdata->parent->manager != NULL)
 				glade_fixed_manager_remove_child
 					(cdata->parent->manager, cdata->widget);
@@ -877,14 +948,10 @@ glade_command_delete_execute (GladeCommandCreateDelete *me)
 		}
 
 		if (me->from_clipboard == TRUE) 
-		{
 			wlist = g_list_prepend (wlist, cdata->widget);
-		}
 		else
-		{
 			glade_project_remove_object 
 				(GLADE_PROJECT (cdata->widget->project), cdata->widget->object);
-		}
 
 		glade_widget_hide (cdata->widget);
 	}
@@ -898,7 +965,7 @@ glade_command_delete_execute (GladeCommandCreateDelete *me)
 	return TRUE;
 }
 
-/**
+/*
  * Execute the cmd and revert it.  Ie, after the execution of this
  * function cmd will point to the undo action
  */
@@ -934,7 +1001,13 @@ glade_command_create_delete_finalize (GObject *obj)
 		cdata = list->data;
 		
 		if (cdata->placeholder)
-			g_object_unref (G_OBJECT (cdata->placeholder));
+		{
+			if (cdata->handler_id)
+				g_signal_handler_disconnect (cdata->placeholder,
+							     cdata->handler_id);
+			if (GTK_OBJECT_FLOATING (cdata->placeholder))
+				gtk_widget_destroy (GTK_WIDGET (cdata->placeholder));
+		}
 
 		if (cdata->widget)
 			g_object_unref (G_OBJECT (cdata->widget));
@@ -958,18 +1031,18 @@ glade_command_create_delete_collapse (GladeCommand *this, GladeCommand *other)
 
 /**
  * glade_command_delete:
- * @widget: a #GladeWidget
+ * @widgets: a #GList of #GladeWidgets
  *
- * TODO: write me
+ * Performs a delete command on the list of widgets
  */
 void
 glade_command_delete (GList *widgets)
 {
-	GladeClipboard           *clipboard = glade_default_app_get_clipboard();
+	GladeClipboard           *clipboard = glade_app_get_clipboard();
 	GladeCommandCreateDelete *me;
 	GladeWidget              *widget = NULL;
 	CommandData              *cdata;
-	GList                    *list;
+	GList                    *list, *l;
 
 	g_return_if_fail (widgets != NULL);
 
@@ -977,6 +1050,7 @@ glade_command_delete (GList *widgets)
 	me->create         = FALSE;
 	me->from_clipboard = 
 		(g_list_find (clipboard->selection, widgets->data) != NULL);
+	me->props_recorded = TRUE; // Dont record props in create_execute
 
 
 	/* internal children cannot be deleted. Notify the user. */
@@ -985,7 +1059,7 @@ glade_command_delete (GList *widgets)
 		widget = list->data;
 		if (widget->internal)
 		{
-			glade_util_ui_message (glade_default_app_get_window(),
+			glade_util_ui_message (glade_app_get_window(),
 					       GLADE_UI_WARN,
 					       _("You cannot delete a widget internal to a composite widget."));
 			return;
@@ -1000,18 +1074,30 @@ glade_command_delete (GList *widgets)
 		cdata->widget  = g_object_ref (G_OBJECT (widget));
 		cdata->parent  = glade_widget_get_parent (widget);
 
+		if (widget->internal)
+			g_critical ("Internal widget in Delete");
+
 		/* !manager here */
 		if (cdata->parent != NULL &&
 		    cdata->parent->manager == NULL &&
 		    glade_util_gtkcontainer_relation 
 		    (cdata->parent, cdata->widget))
 		{
-			cdata->placeholder = 
-				GLADE_PLACEHOLDER (glade_placeholder_new ());
-			g_object_ref (G_OBJECT (cdata->placeholder));
-			gtk_object_sink (GTK_OBJECT (cdata->placeholder));
+			glade_command_placeholder_connect 
+				(cdata, GLADE_PLACEHOLDER (glade_placeholder_new ()));
 		}
 		me->widgets = g_list_prepend (me->widgets, cdata);
+
+
+		/* Record packing props if not deleted from the clipboard */
+		if (me->from_clipboard == FALSE)
+		{
+			for (l = widget->packing_properties; l; l = l->next)
+				cdata->pack_props = 
+					g_list_prepend (cdata->pack_props,
+							glade_property_dup (GLADE_PROPERTY (l->data),
+									    cdata->widget));
+		}
 	}
 
 	if (g_list_length (widgets) == 1)
@@ -1064,15 +1150,15 @@ glade_command_create (GladeWidgetClass *class,
 
 	cdata         = g_new0 (CommandData, 1);
 	cdata->parent = parent;
-	if ((cdata->placeholder = placeholder) != NULL)
-		g_object_ref (G_OBJECT (placeholder));
+
+	glade_command_placeholder_connect (cdata, placeholder);
 	
 	me->widgets = g_list_append (me->widgets, cdata);
 
 	if (parent && parent->manager != NULL)
 		widget = glade_fixed_manager_create_child (parent->manager, class);
 	else
-		widget = glade_widget_new (parent, class, project);
+		widget = glade_widget_new (parent, class, project, TRUE);
 
 	/* widget may be null, e.g. the user clicked cancel on a query */
 	if ((cdata->widget = widget) == NULL)
@@ -1107,19 +1193,19 @@ typedef enum {
 	GLADE_PASTE
 } GladeCutCopyPasteType;
 
-/**
+/*
  * Cut/Copy/Paste
  *
  * Following is the code to extend the GladeCommand Undo/Redo system to 
  * Clipboard functions.
- **/
+ */
 typedef struct {
 	GladeCommand           parent;
 	GladeProject          *project;
 	GList                 *widgets;
 	GladeCutCopyPasteType  type;
 	gboolean               from_clipboard;
-	gboolean               initial_paste;
+	gboolean               props_recorded;
 } GladeCommandCutCopyPaste;
 
 
@@ -1133,14 +1219,13 @@ GLADE_MAKE_COMMAND (GladeCommandCutCopyPaste, glade_command_cut_copy_paste);
 static gboolean
 glade_command_paste_execute (GladeCommandCutCopyPaste *me)
 {
-	GladeProject       *active_project = 
-		glade_default_app_get_active_project ();
+	GladeProject       *active_project = glade_app_get_project ();
 	CommandData        *cdata;
 	GList              *list, *remove = NULL, *l;
 
 	if (me->widgets)
 	{
-		glade_default_app_selection_clear (FALSE);
+		glade_app_selection_clear (FALSE);
 
 		for (list = me->widgets; list && list->data; list = list->next)
 		{
@@ -1159,19 +1244,20 @@ glade_command_paste_execute (GladeCommandCutCopyPaste *me)
 						 G_OBJECT (cdata->placeholder),
 						 cdata->widget->object);
 				else if (cdata->parent->manager != NULL) 
-				{
 					/* Paste at mouse position only once */
 					glade_fixed_manager_add_child (cdata->parent->manager, cdata->widget,
-								       me->initial_paste == FALSE);
-				}
+								       me->props_recorded == FALSE);
 				else
 				{
+					glade_widget_set_parent (cdata->widget, 
+								 cdata->parent);
+
 					glade_widget_class_container_add
 						(cdata->parent->widget_class,
 						 cdata->parent->object,
 						 cdata->widget->object);
-					glade_widget_set_parent (cdata->widget, 
-								 cdata->parent);
+
+					glade_widget_set_packing_properties (cdata->widget, cdata->parent);
 				}
 
 				/* Now that we've added, apply any packing props if nescisary. */
@@ -1187,11 +1273,15 @@ glade_command_paste_execute (GladeCommandCutCopyPaste *me)
 					glade_property_set_value (widget_prop, &value);
 					g_value_unset (&value);
 				}
-
 				
-				if (me->initial_paste == FALSE) 
+				if (me->props_recorded == FALSE) 
 				{
-					// Save the packing properties after the initial paste.
+					/* Save the packing properties after the initial paste.
+					 * (this will be the defaults returned by the container
+					 * implementation after initially adding them).
+					 *
+					 * Otherwise this recorded marker was set when cutting
+					 */
 					g_assert (cdata->pack_props == NULL);
 					for (l = cdata->widget->packing_properties; l; l = l->next)
 						cdata->pack_props = 
@@ -1199,9 +1289,12 @@ glade_command_paste_execute (GladeCommandCutCopyPaste *me)
 									glade_property_dup (GLADE_PROPERTY (l->data),
 											    cdata->widget));
 
-				}
 
-				me->initial_paste = TRUE; // Mark it initialy pasted
+					/* Mark the properties as recorded
+					 */
+					me->props_recorded = TRUE;
+				}
+				
 				
 			}
 
@@ -1216,16 +1309,16 @@ glade_command_paste_execute (GladeCommandCutCopyPaste *me)
 					(me->project, cdata->project, 
 					 cdata->widget->object);
 			
-			glade_default_app_selection_add
+			glade_app_selection_add
 				(cdata->widget->object, FALSE);
 
 			glade_widget_show (cdata->widget);
 		}
-		glade_default_app_selection_changed ();
+		glade_app_selection_changed ();
 
 		if (remove)
 		{
-			glade_clipboard_remove (glade_default_app_get_clipboard(), remove);
+			glade_clipboard_remove (glade_app_get_clipboard(), remove);
 			g_list_free (remove);
 		}
 	}
@@ -1268,7 +1361,7 @@ glade_command_cut_execute (GladeCommandCutCopyPaste *me)
 
 	if (add)
 	{
-		glade_clipboard_add (glade_default_app_get_clipboard(), add);
+		glade_clipboard_add (glade_app_get_clipboard(), add);
 		g_list_free (add);
 	}
 	return TRUE;
@@ -1284,7 +1377,7 @@ glade_command_copy_execute (GladeCommandCutCopyPaste *me)
 
 	if (add)
 	{
-		glade_clipboard_add (glade_default_app_get_clipboard(), add);
+		glade_clipboard_add (glade_app_get_clipboard(), add);
 		g_list_free (add);
 	}
 	return TRUE;
@@ -1300,13 +1393,13 @@ glade_command_copy_undo (GladeCommandCutCopyPaste *me)
 
 	if (remove)
 	{
-		glade_clipboard_remove (glade_default_app_get_clipboard(), remove);
+		glade_clipboard_remove (glade_app_get_clipboard(), remove);
 		g_list_free (remove);
 	}
 	return TRUE;
 }
 
-/**
+/*
  * Execute the cmd and revert it.  Ie, after the execution of this
  * function cmd will point to the undo action
  */
@@ -1366,7 +1459,14 @@ glade_command_cut_copy_paste_finalize (GObject *obj)
 		if (cdata->widget)
 			g_object_unref (cdata->widget);
 		if (cdata->placeholder)
-			g_object_unref (cdata->placeholder);
+		{
+			if (cdata->handler_id)
+				g_signal_handler_disconnect (cdata->placeholder,
+							     cdata->handler_id);
+
+			if (GTK_OBJECT_FLOATING (cdata->placeholder))
+				gtk_widget_destroy (GTK_WIDGET (cdata->placeholder));
+		}
 		if (cdata->pack_props)
 		{
 			g_list_foreach (cdata->pack_props, (GFunc)g_object_unref, NULL);
@@ -1405,74 +1505,25 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 	g_return_if_fail (widgets && widgets->data);
 	g_return_if_fail (parent == NULL || GLADE_IS_WIDGET (parent));
 
-
-	/* Some prelimenary error checking here */
-	switch (type) {
-	case GLADE_CUT:
-	case GLADE_COPY:
-		for (list = widgets; list && list->data; list = list->next)
-		{
-			widget = list->data;
-			if (widget->internal)
-			{
-				glade_util_ui_message 
-					(glade_default_app_get_window(), 
-					 GLADE_UI_WARN,
-					 _("You cannot copy/cut a widget "
-					   "internal to a composite widget."));
-				return;
-			}
-		}
-		fmt = (type == GLADE_CUT) ? _("Cut %s") : _("Copy %s");
-		break;
-	case GLADE_PASTE:
-
-		for (list = widgets; 
-		     list && list->data; 
-		     list = list->next)
-		{
-			widget = list->data;
-			if (parent == NULL &&
-			    GTK_WIDGET_TOPLEVEL (widget->object) == FALSE)
-			{
-				glade_util_ui_message 
-					(glade_default_app_get_window(), 
-					 GLADE_UI_WARN,
-					 _("Only top-level widgets can be "
-					   "pasted without a parent."));
-				return;
-			}
-
-			if (placeholder &&
-			    GTK_WIDGET_TOPLEVEL (widget->object))
-			{
-				glade_util_ui_message 
-					(glade_default_app_get_window(), 
-					 GLADE_UI_WARN,
-					 _("You cannot paste a top-level "
-					   "to a placeholder."));
-				return;
-			}
-		}
-		
-		fmt = _("Paste %s");
-		break;
-	}
-	g_assert (fmt);
-	
+	/* Things can go wrong in this function if the dataset is inacurate,
+	 * we make no real attempt here to recover, just g_critical() and
+	 * fix the bugs as they pop up.
+	 */
+	fmt = 
+		(type == GLADE_CUT) ? _("Cut %s") : 
+		(type == GLADE_COPY) ? _("Copy %s") : _("Paste %s");
 	widget = GLADE_WIDGET (widgets->data);
-
-	/* And now we feel safe enough to go on and create */
 	me = (GladeCommandCutCopyPaste *) 
 		g_object_new (GLADE_COMMAND_CUT_COPY_PASTE_TYPE, NULL);
 
-	if (type == GLADE_PASTE && placeholder)
+	if (type == GLADE_PASTE && placeholder && 
+	    GTK_IS_WINDOW (widget->object) == FALSE)
 	{
 		GladeWidget *some_widget = glade_placeholder_get_parent (placeholder);
-		me->project = some_widget->project;
+		me->project = glade_widget_get_project (some_widget);
 	}
 	else if (type == GLADE_PASTE)
-		me->project = glade_default_app_get_active_project();
+		me->project = glade_app_get_project();
 	else
 		me->project = glade_widget_get_project (widget);
 
@@ -1488,6 +1539,9 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 
 		cdata              = g_new0 (CommandData, 1);
 
+		if (widget->internal)
+			g_critical ("Internal widget in Cut/Copy/Paste");
+
 		/* Widget */
 		if (type == GLADE_COPY)
 		{
@@ -1502,11 +1556,16 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 		/* Parent */
 		if (parent == NULL)
 			cdata->parent = glade_widget_get_parent (widget);
-		else if (type == GLADE_PASTE && placeholder)
+		else if (type == GLADE_PASTE && placeholder && 
+			 GTK_IS_WINDOW (widget->object) == FALSE)
 			cdata->parent = glade_placeholder_get_parent (placeholder);
-		else if (GTK_WIDGET_TOPLEVEL (widget->object) == FALSE)
+		else if (GTK_IS_WINDOW (widget->object) == FALSE)
 			cdata->parent = parent;
 
+		if (cdata->parent == NULL && type == GLADE_PASTE &&
+		    GTK_IS_WINDOW (widget->object) == FALSE)
+			g_critical ("Parentless non GtkWindow widget in Paste");
+
 		/* Placeholder */
 		if (type == GLADE_CUT)
 		{
@@ -1514,18 +1573,16 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 			    glade_util_gtkcontainer_relation
 			    (cdata->parent, cdata->widget))
 			{
-				cdata->placeholder = GLADE_PLACEHOLDER 
-					(glade_placeholder_new ());
-				g_object_ref (G_OBJECT (cdata->placeholder));
-				gtk_object_sink (GTK_OBJECT (cdata->placeholder));
+				glade_command_placeholder_connect
+					(cdata, GLADE_PLACEHOLDER (glade_placeholder_new ()));
 			}
 			else if (placeholder != NULL)
-				cdata->placeholder = g_object_ref (placeholder);
+				glade_command_placeholder_connect (cdata, placeholder);
 		}
 		else if (type == GLADE_PASTE && placeholder != NULL &&
 			 g_list_length (widgets) == 1)
 		{
-			cdata->placeholder = g_object_ref (placeholder);
+			glade_command_placeholder_connect (cdata, placeholder);
 		}
 		else if (type == GLADE_PASTE && cdata->parent &&
 			 cdata->parent->manager == NULL &&
@@ -1543,7 +1600,8 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 					    g_list_find (placeholders, child) == NULL)
 					{
 						placeholders = g_list_append (placeholders, child);
-						cdata->placeholder = g_object_ref (child);
+						glade_command_placeholder_connect
+							(cdata, GLADE_PLACEHOLDER (child));
 						break;
 					}
 				}
@@ -1556,6 +1614,11 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 		 */
 		if (type == GLADE_CUT)
 		{
+			/* We dont want the paste mechanism to overwrite our 
+			 * packing props when we undo a CUT command; so we have to 
+			 * mark them "recorded" too.
+			 */
+			me->props_recorded = TRUE;
 			for (l = cdata->widget->packing_properties; l; l = l->next)
 				cdata->pack_props = 
 					g_list_prepend (cdata->pack_props,
@@ -1574,11 +1637,6 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 		me->widgets = g_list_prepend (me->widgets, cdata);
 	}
 
-	if (type == GLADE_CUT)
-		me->initial_paste = TRUE;
-	else
-		me->initial_paste = FALSE;
-
 	glade_command_check_group (GLADE_COMMAND (me));
 
 	/*
@@ -1586,7 +1644,7 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 	 */
 	if (glade_command_cut_copy_paste_execute (GLADE_COMMAND (me)))
 		glade_command_push_undo 
-			(glade_default_app_get_active_project(),
+			(glade_app_get_project(),
 			 GLADE_COMMAND (me));
 	else
 		g_object_unref (G_OBJECT (me));
@@ -1597,11 +1655,13 @@ glade_command_cut_copy_paste_common (GList                 *widgets,
 
 /**
  * glade_command_paste:
- * @widgets: a #GList
+ * @widgets: a #GList of #GladeWidget
  * @parent: a #GladeWidget
  * @placeholder: a #GladePlaceholder
  *
- * TODO: write me
+ * Performs a paste command on all widgets in @widgets to @parent, possibly
+ * replacing @placeholder (note toplevels dont need a parent; the active project
+ * will be used when pasting toplevel objects).
  */
 void
 glade_command_paste (GList *widgets, GladeWidget *parent, GladePlaceholder *placeholder)
