@@ -1,6 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * glade-clipboard.c - An object for handling Cut/Copy/Paste.
+ * glade-fixed-manager.c - A proxy object used as an interface to
+ *                         handle child placement in fixed type GtkContainer 
+ *                         such as GtkFixed and GtkLayout.
  *
  * Copyright (C) 2005 The GNOME Foundation.
  *
@@ -365,7 +367,7 @@ glade_fixed_manager_create_child_impl (GladeFixedManager *manager,
 	GladeWidget *gwidget =
 		glade_widget_new (manager->container,
 				  wclass, 
-				  manager->container->project);
+				  manager->container->project, TRUE);
 
 	return gwidget;
 }
@@ -596,8 +598,8 @@ glade_fixed_manager_event (GtkWidget         *widget,
 	{
 	case GDK_BUTTON_PRESS: // add widget
 		
-		add_class = glade_default_app_get_add_class ();
-		alt_class = glade_default_app_get_alt_class ();
+		add_class = glade_app_get_add_class ();
+		alt_class = glade_app_get_alt_class ();
 
 		if (((GdkEventButton *)event)->button == 1)
 		{
@@ -621,14 +623,14 @@ glade_fixed_manager_event (GtkWidget         *widget,
 				
 				/* reset the palette */
 				glade_palette_unselect_widget 
-					(glade_default_app_get_palette ());
+					(glade_app_get_palette ());
 				handled = TRUE;
 			}
 		}
 		break;
 	case GDK_ENTER_NOTIFY:
 	case GDK_MOTION_NOTIFY:
-		if (glade_default_app_get_add_class ())
+		if (glade_app_get_add_class ())
 			glade_cursor_set (((GdkEventCrossing *)event)->window, GLADE_CURSOR_ADD_WIDGET);
 		else
 			glade_cursor_set (((GdkEventCrossing *)event)->window, GLADE_CURSOR_SELECTOR);
