@@ -38,6 +38,102 @@
 #include "glade-editor-property.h"
 #include "glade-debug.h"
 
+
+/* Hardcoded recognized atk actions
+ */
+typedef struct {
+	gchar *prop_name;
+	gchar *id;
+	gchar *name;
+	gchar *tooltip;
+} GPCAtkPropertyTab;
+
+static const GPCAtkPropertyTab action_names_table[] = {
+	{ "click",   "atk-click",   N_("Click"),   
+	  N_("Set the desctription of the Click atk action") },
+	{ "press",   "atk-press",   N_("Press"),   
+	  N_("Set the desctription of the Press atk action") },
+	{ "release", "atk-release", N_("Release"), 
+	  N_("Set the desctription of the Release atk action") },
+	{ "activate", "atk-activate", N_("Activate"), 
+	  N_("Set the desctription of the Activate atk action") }
+};
+
+static const GPCAtkPropertyTab relation_names_table[] = {
+	{ "controlled-by", "atk-controlled-by", N_("Controlled By"),
+	  N_("Indicates an object controlled by one or more target objects") },
+
+	{ "controlled-for", "atk-controlled-for", N_("Controlled For"),
+	  N_("Indicates an object is an controller for one or more target objects") },
+
+	{ "labelled-by", "atk-labelled-by", N_("Labelled By"),
+	  N_("Indicates an object is labelled by one or more target objects") },
+
+	{ "label-for", "atk-label-for", N_("Label For"),
+	  N_("Indicates an object is a label for one or more target objects") },
+
+	{ "member-of", "atk-member-of", N_("Member Of"),
+	  N_("Indicates an object is a member of a group of one or more target objects") },
+
+	{ "child-node-of", "atk-child-node-of", N_("Child Node Of"),
+	  N_("Indicates an object is a cell in a treetable which is displayed "
+	     "because a cell in the same column is expanded and identifies that cell") },
+
+	{ "flows-to", "atk-flows-to", N_("Flows To"),
+	  N_("Indicates that the object has content that flows logically to another "
+	     "AtkObject in a sequential way, (for instance text-flow)") },
+
+	{ "flows-from", "atk-flows-from", N_("Flows From"),
+	  N_("Indicates that the object has content that flows logically from another "
+	     "AtkObject in a sequential way, (for instance text-flow)") },
+
+	{ "subwindow-of", "atk-subwindow-of", N_("Subwindow Of"),
+	  N_("Indicates a subwindow attached to a component but otherwise has no "
+	     "connection in  the UI heirarchy to that component") },
+
+	{ "embeds", "atk-embeds", N_("Embeds"),
+	  N_("Indicates that the object visually embeds another object's content, "
+	     "i.e. this object's content flows around another's content") },
+
+	{ "embedded-by", "atk-embedded-by", N_("Embedded By"),
+	  N_("Inverse of 'Embeds', indicates that this object's content "
+	     "is visualy embedded in another object") },
+
+	{ "popup-for", "atk-popup-for", N_("Popup For"),
+	  N_("Indicates that an object is a popup for another object") },
+
+	{ "parent-window-of", "atk-parent-window-of", N_("Parent Window Of"),
+	  N_("Indicates that an object is a parent window of another object") }
+};
+
+
+/**
+ * glade_property_class_atk_realname:
+ * @atk_name: The id of the atk property
+ *
+ * Translates a GladePropertyClass->id to the name that should be
+ * saved into the glade file.
+ *
+ * Returns: a pointer to a constant string.
+ */
+G_CONST_RETURN gchar *
+glade_property_class_atk_realname (const gchar *atk_name)
+{
+	gint i;
+
+	g_return_val_if_fail (atk_name != NULL, NULL);
+
+	for (i = 0; i < G_N_ELEMENTS (action_names_table); i++)
+		if (!strcmp (action_names_table[i].id, atk_name))
+			return action_names_table[i].prop_name;
+
+	for (i = 0; i < G_N_ELEMENTS (relation_names_table); i++)
+		if (!strcmp (relation_names_table[i].id, atk_name))
+			return relation_names_table[i].prop_name;
+
+	return atk_name;
+}
+
 /**
  * glade_property_class_new:
  *
@@ -59,7 +155,7 @@ glade_property_class_new (void)
 	property_class->displayable_values = NULL;
 	property_class->query = FALSE;
 	property_class->optional = FALSE;
-	property_class->optional_default = TRUE;
+	property_class->optional_default = FALSE;
 	property_class->common = FALSE;
 	property_class->packing = FALSE;
 	property_class->is_modified = FALSE;
@@ -70,7 +166,8 @@ glade_property_class_new (void)
 	property_class->save = TRUE;
 	property_class->ignore = FALSE;
 	property_class->resource = FALSE;
-	property_class->translatable = TRUE;
+	property_class->translatable = FALSE;
+	property_class->atk_type = GPC_ATK_NONE;
 	return property_class;
 }
 
@@ -149,42 +246,42 @@ glade_property_class_clone (GladePropertyClass *property_class)
 
 /**
  * glade_property_class_free:
- * @class: a #GladePropertyClass
+ * @property_class: a #GladePropertyClass
  *
  * Frees @class and its associated memory.
  */
 void
-glade_property_class_free (GladePropertyClass *class)
+glade_property_class_free (GladePropertyClass *property_class)
 {
-	if (class == NULL)
+	if (property_class == NULL)
 		return;
 
-	g_return_if_fail (GLADE_IS_PROPERTY_CLASS (class));
+	g_return_if_fail (GLADE_IS_PROPERTY_CLASS (property_class));
 
-	g_free (class->id);
-	g_free (class->tooltip);
-	g_free (class->name);
-	if (class->orig_def)
+	g_free (property_class->id);
+	g_free (property_class->tooltip);
+	g_free (property_class->name);
+	if (property_class->orig_def)
 	{
-		if (G_VALUE_TYPE (class->orig_def) != 0)
-			g_value_unset (class->orig_def);
-		g_free (class->orig_def);
+		if (G_VALUE_TYPE (property_class->orig_def) != 0)
+			g_value_unset (property_class->orig_def);
+		g_free (property_class->orig_def);
 	}
-	if (class->def)
+	if (property_class->def)
 	{
-		if (G_VALUE_TYPE (class->def) != 0)
-			g_value_unset (class->def);
-		g_free (class->def);
+		if (G_VALUE_TYPE (property_class->def) != 0)
+			g_value_unset (property_class->def);
+		g_free (property_class->def);
 	}
-	g_list_foreach (class->parameters, (GFunc) glade_parameter_free, NULL);
-	g_list_free (class->parameters);
+	g_list_foreach (property_class->parameters, (GFunc) glade_parameter_free, NULL);
+	g_list_free (property_class->parameters);
 	
-	if (class->displayable_values)
+	if (property_class->displayable_values)
 	{
 		gint i, len;
 		GArray *disp_val;
 		
-		disp_val = class->displayable_values;
+		disp_val = property_class->displayable_values;
 		len = disp_val->len;
 		
 		for (i = 0; i < len; i++)
@@ -201,7 +298,7 @@ glade_property_class_free (GladePropertyClass *class)
 		g_array_free(disp_val, TRUE);
 	}	
 	
-	g_free (class);
+	g_free (property_class);
 }
 
 
@@ -236,17 +333,7 @@ glade_property_class_make_string_from_enum (GType etype, gint eval)
 	return string;
 }
 
-/**
- * glade_property_class_make_string_from_flags:
- * @class: A GFlagsClass property class.
- * @fvals: The flags to include in the string.
- * @displayables: if TRUE it will try to use diplayable values.
- *
- * Create a string with the flags wich are set in @fvals.
- *
- * Returns: a newly allocated string.
-*/
-gchar *
+static gchar *
 glade_property_class_make_string_from_flags (GladePropertyClass *class, guint fvals, gboolean displayables)
 {
 	GFlagsClass *fclass;
@@ -271,6 +358,9 @@ glade_property_class_make_string_from_flags (GladePropertyClass *class, guint fv
 			g_string_append(string, " | ");
 		
 		g_string_append(string, (val_str) ? val_str : fvalue->value_name);
+		
+		/* If one of the flags value is 0 this loop become infinite :) */
+		if (fvalue->value == 0) break;
 	}
 	
 	retval = string->str;
@@ -281,21 +371,80 @@ glade_property_class_make_string_from_flags (GladePropertyClass *class, guint fv
 	return retval;
 }
 
+static gchar *
+glade_property_class_make_string_from_object (GladePropertyClass *property_class,
+					      GObject            *object)
+{
+	GladeWidget *gwidget;
+	gchar       *string = NULL, *filename;
+
+	if (!object) return NULL;
+
+	if (property_class->pspec->value_type == GDK_TYPE_PIXBUF)
+	{
+		if ((filename = g_object_get_data (object, "GladeFileName")) != NULL)
+			string = g_path_get_basename (filename);
+	}
+	else if (property_class->pspec->value_type == GTK_TYPE_ADJUSTMENT)
+	{
+		GtkAdjustment *adj = GTK_ADJUSTMENT (object);
+		string = g_strdup_printf ("%lf %lf %lf %lf %lf %lf", 
+					  adj->value, adj->lower, adj->upper, 
+					  adj->step_increment, 
+					  adj->page_increment,
+					  adj->page_size);
+	}
+	else if ((gwidget = glade_widget_get_from_gobject (object)) != NULL)
+		string = g_strdup (gwidget->name);
+	else
+		g_critical ("Object type property refers to an object "
+			    "outside the project");
+	
+	return string;
+}
+
+static gchar *
+glade_property_class_make_string_from_objects (GladePropertyClass *property_class,
+					       GList              *objects)
+{
+	GObject *object;
+	GList   *list;
+	gchar   *string = NULL, *obj_str, *tmp;
+
+	for (list = objects; list; list = list->next)
+	{
+		object = list->data;
+
+		obj_str = glade_property_class_make_string_from_object 
+			(property_class, object);
+
+		if (string == NULL)
+			string = obj_str;
+		else if (obj_str != NULL)
+		{
+			tmp = g_strdup_printf ("%s%s%s", string, GPC_OBJECT_DELIMITER, obj_str);
+			string = (g_free (string), tmp);
+			g_free (obj_str);
+		}
+	}
+	return string;
+}
+
 /**
  * glade_property_class_make_string_from_gvalue:
- * @property_class:
- * @value:
+ * @property_class: A #GladePropertyClass
+ * @value: A #GValue
  *
- * TODO: write me
- *
- * Returns:
+ * Returns: A newly allocated string representation of @value
  */
 gchar *
 glade_property_class_make_string_from_gvalue (GladePropertyClass *property_class,
 					      const GValue *value)
 {
 	gchar    *string = NULL, **strv;
+	GObject  *object;
 	GdkColor *color;
+	GList    *objects;
 
 	if (G_IS_PARAM_SPEC_ENUM(property_class->pspec))
 	{
@@ -365,27 +514,15 @@ glade_property_class_make_string_from_gvalue (GladePropertyClass *property_class
 					  GLADE_TAG_TRUE : GLADE_TAG_FALSE);
 	else if (G_IS_PARAM_SPEC_OBJECT(property_class->pspec))
 	{
-		GObject *object = g_value_get_object (value);
-
-		if (property_class->pspec->value_type == GDK_TYPE_PIXBUF)
-		{
-			gchar *filename;
-
-			if (object && (filename = g_object_get_data
-				       (object, "GladeFileName")) != NULL)
-				string = g_path_get_basename (filename);
-		}
-		else
-		{
-			GladeWidget *gwidget;
-			if (object)
-			{
-				if ((gwidget = glade_widget_get_from_gobject (object)) != NULL)
-					string = g_strdup (gwidget->name);
-				else
-					g_critical ("Object type property refers to an object outside the project");
-			}
-		}
+		object = g_value_get_object (value);
+		string = glade_property_class_make_string_from_object
+			(property_class, object);
+	}
+	else if (GLADE_IS_PARAM_SPEC_OBJECTS (property_class->pspec))
+	{
+		objects = g_value_get_boxed (value);
+		string = glade_property_class_make_string_from_objects
+			(property_class, objects);
 	}
 	else
 		g_critical ("Unsupported pspec type %s",
@@ -395,9 +532,6 @@ glade_property_class_make_string_from_gvalue (GladePropertyClass *property_class
 }
 
 /* This is copied exactly from libglade. I've just renamed the function.
- *
- * TODO: Expose function from libglade and use the one in libglade (once
- * Ivan Wongs libglade patch gets in...).
  */
 static guint
 glade_property_class_make_flags_from_string (GType type, const char *string)
@@ -501,6 +635,73 @@ glade_property_class_make_enum_from_string (GType type, const char *string)
     return ret;
 }
 
+static GObject *
+glade_property_class_make_object_from_string (GladePropertyClass *property_class,
+					      const gchar        *string,
+					      GladeProject       *project)
+{
+	GObject *object = NULL;
+	gchar   *fullpath;
+
+	if (string == NULL) return NULL;
+	
+	if (property_class->pspec->value_type == GDK_TYPE_PIXBUF && project)
+	{
+		GdkPixbuf *pixbuf;
+
+		fullpath = glade_project_resource_fullpath (project, string);
+
+		if ((pixbuf = gdk_pixbuf_new_from_file (fullpath, NULL)) != NULL)
+		{
+			g_object_set_data_full (G_OBJECT(pixbuf), 
+						"GladeFileName",
+						g_strdup (string),
+						g_free);
+
+			object = G_OBJECT(pixbuf);
+		}
+		g_free (fullpath);
+	}
+	if (property_class->pspec->value_type == GTK_TYPE_ADJUSTMENT)
+	{
+		gdouble value, lower, upper, step_increment, page_increment, page_size;
+		
+		sscanf (string, "%lf %lf %lf %lf %lf %lf", &value, &lower, &upper, &step_increment, &page_increment, &page_size);
+		object = G_OBJECT (gtk_adjustment_new (value, lower, upper, step_increment, page_increment, page_size));
+	}
+	else
+	{
+		GladeWidget *gwidget;
+		if ((gwidget = glade_project_get_widget_by_name 
+			       (project, string)) != NULL)
+			object = gwidget->object;
+	}
+	
+	return object;
+}
+
+static GList *
+glade_property_class_make_objects_from_string (GladePropertyClass *property_class,
+					       const gchar        *string,
+					       GladeProject       *project)
+{
+	GList    *objects = NULL;
+	GObject  *object;
+	gchar   **split;
+	guint     i;
+	
+	if ((split = g_strsplit (string, GPC_OBJECT_DELIMITER, 0)) != NULL)
+	{
+		for (i = 0; split[i]; i++)
+		{
+			if ((object = glade_property_class_make_object_from_string
+			     (property_class, split[i], project)) != NULL)
+				objects = g_list_prepend (objects, object);
+		}
+		g_strfreev (split);
+	}
+	return objects;
+}
 
 /**
  * glade_property_class_make_gvalue_from_string:
@@ -602,38 +803,15 @@ glade_property_class_make_gvalue_from_string (GladePropertyClass *property_class
 	}
 	else if (G_IS_PARAM_SPEC_OBJECT(property_class->pspec))
 	{
-		if (property_class->pspec->value_type == GDK_TYPE_PIXBUF && project)
-		{
-			GdkPixbuf *pixbuf;
-
-			if (string)
-			{
-				fullpath = 
-					glade_project_resource_fullpath (project,
-									 string);
-				if ((pixbuf = gdk_pixbuf_new_from_file
-				     (fullpath, NULL)) != NULL)
-				{
-					g_object_set_data_full (G_OBJECT(pixbuf), 
-								"GladeFileName",
-								g_strdup (string),
-								g_free);
-					g_value_take_object (value, G_OBJECT(pixbuf));
-				}
-				g_free (fullpath);
-			}
-			else
-				g_value_set_object (value, NULL);
-		}
-		else
-		{
-			GladeWidget *gwidget;
-			if (string && (gwidget = glade_project_get_widget_by_name 
-				       (project, string)) != NULL)
-				g_value_set_object (value, gwidget->object);
-			else
-				g_value_set_object (value, NULL);
-		}
+		GObject *object = glade_property_class_make_object_from_string
+			(property_class, string, project);
+		g_value_set_object (value, object);
+	}
+	else if (GLADE_IS_PARAM_SPEC_OBJECTS (property_class->pspec))
+	{
+		GList *objects = glade_property_class_make_objects_from_string
+			(property_class, string, project);
+		g_value_set_boxed (value, objects);
 	}
 	else
 		g_critical ("Unsupported pspec type %s",
@@ -642,6 +820,15 @@ glade_property_class_make_gvalue_from_string (GladePropertyClass *property_class
 	return value;
 }
 
+/**
+ * glade_property_class_make_gvalue_from_vl:
+ * @property_class: A #GladePropertyClass
+ * @vl: a #va_list holding one argument of the correct type
+ *      specified by @property_class
+ *
+ * Returns: A #GValue created based on the @property_class
+ *          and a @vl arg of the correct type.
+ */
 GValue *
 glade_property_class_make_gvalue_from_vl (GladePropertyClass  *class,
 					  va_list              vl)
@@ -687,14 +874,50 @@ glade_property_class_make_gvalue_from_vl (GladePropertyClass  *class,
 		g_value_set_object (value, va_arg (vl, gpointer));
 	else if (G_IS_PARAM_SPEC_BOXED(class->pspec))
 		g_value_set_boxed (value, va_arg (vl, gpointer));
+	else if (GLADE_IS_PARAM_SPEC_OBJECTS(class->pspec))
+		g_value_set_boxed (value, va_arg (vl, gpointer));
 	else
 		g_critical ("Unsupported pspec type %s",
-			    g_type_name(class->pspec->value_type));
+			    g_type_name(G_PARAM_SPEC_TYPE (class->pspec)));
 	
 	return value;
 }
 
+/**
+ * glade_property_class_make_gvalue:
+ * @class: A #GladePropertyClass
+ * @...: an argument of the correct type specified by @property_class
+ *
+ * Returns: A #GValue created based on the @property_class
+ *          and the provided argument.
+ */
+GValue *
+glade_property_class_make_gvalue (GladePropertyClass  *class,
+				  ...)
+{
+	GValue  *value;
+	va_list  vl;
 
+	g_return_val_if_fail (class != NULL, NULL);
+
+	va_start (vl, class);
+	value = glade_property_class_make_gvalue_from_vl (class, vl);
+	va_end (vl);
+
+	return value;
+}
+
+
+/**
+ * glade_property_class_set_vl_from_gvalue:
+ * @class: A #GladePropertyClass
+ * @value: A #GValue to set
+ * @vl: a #va_list holding one argument of the correct type
+ *      specified by @class
+ * 
+ *
+ * Sets @vl from @value based on @class criteria.
+ */
 void
 glade_property_class_set_vl_from_gvalue (GladePropertyClass  *class,
 					 GValue              *value,
@@ -740,19 +963,92 @@ glade_property_class_set_vl_from_gvalue (GladePropertyClass  *class,
 		*(gpointer *)(va_arg (vl, gpointer *)) = g_value_get_object (value);
 	else if (G_IS_PARAM_SPEC_BOXED(class->pspec))
 		*(gpointer *)(va_arg (vl, gpointer *)) = g_value_get_boxed (value);
+	else if (GLADE_IS_PARAM_SPEC_OBJECTS(class->pspec))
+		*(gpointer *)(va_arg (vl, gpointer *)) = g_value_get_boxed (value);
 	else
 		g_critical ("Unsupported pspec type %s",
-			    g_type_name(class->pspec->value_type));
+			    g_type_name(G_PARAM_SPEC_TYPE (class->pspec)));
+}
+
+/**
+ * glade_property_class_get_from_gvalue:
+ * @class: A #GladePropertyClass
+ * @value: A #GValue to set
+ * @...: a return location of the correct type
+ * 
+ *
+ * Assignes the provided return location to @value
+ */
+void
+glade_property_class_get_from_gvalue (GladePropertyClass  *class,
+				      GValue              *value,
+				      ...)
+{
+	va_list  vl;
+
+	g_return_if_fail (class != NULL);
+
+	va_start (vl, value);
+	glade_property_class_set_vl_from_gvalue (class, value, vl);
+	va_end (vl);
+}
+
+/**
+ * glade_property_class_list_atk_relations:
+ * @owner_type: The #GType of the owning widget class.
+ *
+ * Returns: a #GList of newly created atk relation #GladePropertyClass.
+ */
+GList *
+glade_property_class_list_atk_relations (GType owner_type)
+{
+	const GPCAtkPropertyTab *relation_tab = NULL;
+	GladePropertyClass      *property_class;
+	GList                   *list = NULL;
+	gint                     i;
+	
+	/* Loop through our hard-coded table enties */
+	for (i = 0; i < G_N_ELEMENTS (relation_names_table); i++)
+	{
+		relation_tab = &relation_names_table[i];
+
+		property_class                    = glade_property_class_new ();
+		property_class->pspec             = 
+			glade_param_spec_objects (relation_tab->id,
+						  _(relation_tab->name),
+						  _(relation_tab->tooltip),
+						  ATK_TYPE_IMPLEMENTOR,
+						  G_PARAM_READWRITE);
+		
+		property_class->pspec->owner_type = owner_type;
+		property_class->id                = g_strdup (relation_tab->id);
+		property_class->name              = g_strdup (_(relation_tab->name));
+		property_class->tooltip           = g_strdup (_(relation_tab->tooltip));
+		property_class->atk_type          = GPC_ATK_RELATION;
+		property_class->visible_lines     = 2;
+		property_class->ignore            = TRUE;
+
+		property_class->def = 
+			glade_property_class_make_gvalue_from_string
+			(property_class, "", NULL);
+		
+		property_class->orig_def = 
+			glade_property_class_make_gvalue_from_string
+			(property_class, "", NULL);
+
+		list = g_list_prepend (list, property_class);
+	}
+
+	return g_list_reverse (list);
 }
 
 
 /**
  * glade_property_class_new_from_spec:
- * @spec:
+ * @spec: A #GParamSpec
  *
- * TODO: write me
- *
- * Returns:
+ * Returns: a newly created #GladePropertyClass based on @spec
+ *          or %NULL if its unsupported.
  */
 GladePropertyClass *
 glade_property_class_new_from_spec (GParamSpec *spec)
@@ -766,11 +1062,15 @@ glade_property_class_new_from_spec (GParamSpec *spec)
 	property_class = glade_property_class_new ();
 
 	property_class->pspec = spec;
+	
+	/* We only use the writable properties */
+	if ((spec->flags & G_PARAM_WRITABLE) == 0)
+		goto failed;
 
 	/* Register only editable properties.
 	 */
 	if (!glade_editor_property_supported (property_class->pspec))
-		goto lblError;
+		goto failed;
 	
 	property_class->id   = g_strdup (spec->name);
 	property_class->name = g_strdup (g_param_spec_get_nick (spec));
@@ -783,19 +1083,36 @@ glade_property_class_new_from_spec (GParamSpec *spec)
 					  g_param_spec_get_name (spec)) != NULL)
 		property_class->common = TRUE;
 
+	/* Flag the construct only properties */
+	if (spec->flags & G_PARAM_CONSTRUCT_ONLY)
+		property_class->construct_only = TRUE;
+
+	if (g_type_is_a (spec->owner_type, ATK_TYPE_OBJECT))
+	{
+		property_class->atk_type     = GPC_ATK_PROPERTY;
+		property_class->ignore       = TRUE;
+
+		/* We only use the name and desctription props,
+		 * they are both translatable.
+		 */
+		property_class->translatable = TRUE;
+	}
+
 	if (!property_class->id || !property_class->name)
 	{
-		g_warning ("Failed to create property class from spec");
-		goto lblError;
+		g_critical ("No name or id for "
+			    "glade_property_class_new_from_spec, failed.");
+		goto failed;
 	}
 
 	property_class->tooltip  = g_strdup (g_param_spec_get_blurb (spec));
 	property_class->orig_def = glade_property_class_get_default_from_spec (spec);
 	property_class->def      = glade_property_class_get_default_from_spec (spec);
 
+	g_type_class_unref (gtk_widget_class);
 	return property_class;
 
-  lblError:
+  failed:
 	glade_property_class_free (property_class);
 	g_type_class_unref (gtk_widget_class);
 	return NULL;
@@ -815,7 +1132,6 @@ glade_property_class_is_visible (GladePropertyClass *class)
 	return class->visible;
 }
 
-
 /**
  * glade_property_class_is_object:
  * @property_class: A #GladePropertyClass
@@ -828,8 +1144,11 @@ gboolean
 glade_property_class_is_object (GladePropertyClass  *class)
 {
 	g_return_val_if_fail (GLADE_IS_PROPERTY_CLASS (class), FALSE);
-	return (G_IS_PARAM_SPEC_OBJECT(class->pspec) &&
-		class->pspec->value_type != GDK_TYPE_PIXBUF);
+
+ 	return (GLADE_IS_PARAM_SPEC_OBJECTS (class->pspec) ||
+		(G_IS_PARAM_SPEC_OBJECT(class->pspec) &&
+		 class->pspec->value_type != GDK_TYPE_PIXBUF &&
+		 class->pspec->value_type != GTK_TYPE_ADJUSTMENT));
 }
 
 
@@ -928,7 +1247,6 @@ gpc_get_displayable_values_from_node (GladeXmlNode *node,
 
 /**
  * glade_property_class_make_adjustment:
- *
  * @property_class: a pointer to the property class
  *
  * Creates and appropriate GtkAdjustment for use in the editor
@@ -983,9 +1301,9 @@ glade_property_class_make_adjustment (GladePropertyClass *property_class)
 	} else if (G_IS_PARAM_SPEC_DOUBLE(property_class->pspec))
 	{
 		float_range = TRUE;
-		min = (gdouble)((GParamSpecFloat *) property_class->pspec)->minimum;
-		max = (gdouble)((GParamSpecFloat *) property_class->pspec)->maximum;
-		def = (gdouble)((GParamSpecFloat *) property_class->pspec)->default_value;
+		min = (gdouble)((GParamSpecDouble *) property_class->pspec)->minimum;
+		max = (gdouble)((GParamSpecDouble *) property_class->pspec)->maximum;
+		def = (gdouble)((GParamSpecDouble *) property_class->pspec)->default_value;
 	} else
 	{
 		g_critical ("Can't make adjustment for pspec type %s",
@@ -1002,11 +1320,11 @@ glade_property_class_make_adjustment (GladePropertyClass *property_class)
 
 /**
  * glade_property_class_update_from_node:
- * @node: the <property> node
+ * @node: the property node
  * @module: a #GModule to lookup symbols from the plugin
  * @object_type: the #GType of the owning object
  * @property_class: a pointer to the property class
- * @domain: the domain to translate catalog strings from.
+ * @domain: the domain to translate catalog strings from
  *
  * Updates the @property_class with the contents of the node in the xml
  * file. Only the values found in the xml file are overridden.
@@ -1039,6 +1357,7 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 		return FALSE;
 	g_free (buff);
 
+	
 	/* If Disabled="TRUE" we set *property_class to NULL, but we return TRUE.
 	 * The caller may want to remove this property from its list.
 	 */
@@ -1075,6 +1394,11 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 				g_free (class->def);
 			}
 			class->def = glade_property_class_get_default_from_spec (class->pspec);
+			
+			if (class->orig_def == NULL)
+				class->orig_def =
+					glade_property_class_get_default_from_spec (class->pspec);
+
 		}
 
  		g_free (buff);
@@ -1092,8 +1416,7 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 	}
 
 	/* Get the default */
-	buff = glade_xml_get_property_string (node, GLADE_TAG_DEFAULT);
-	if (buff)
+	if ((buff = glade_xml_get_property_string (node, GLADE_TAG_DEFAULT)) != NULL)
 	{
 		if (class->def) {
 			g_value_unset (class->def);
@@ -1104,16 +1427,14 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 	}
 
 	/* If needed, update the name... */
-	buff = glade_xml_get_property_string (node, GLADE_TAG_NAME);
-	if (buff)
+	if ((buff = glade_xml_get_property_string (node, GLADE_TAG_NAME)) != NULL)
 	{
 		g_free (class->name);
 		class->name = g_strdup (dgettext (domain, buff));
 	}
 	
 	/* ...and the tooltip */
-	buff = glade_xml_get_value_string (node, GLADE_TAG_TOOLTIP);
-	if (buff)
+	if ((buff = glade_xml_get_value_string (node, GLADE_TAG_TOOLTIP)) != NULL)
 	{
 		g_free (class->tooltip);
 		class->tooltip = g_strdup (dgettext (domain, buff));
@@ -1149,15 +1470,14 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 	glade_xml_get_value_int (node, GLADE_TAG_VISIBLE_LINES,  &class->visible_lines);
 
 	/* Get the Parameters */
-	child = glade_xml_search_child (node, GLADE_TAG_PARAMETERS);
-	if (child)
+	if ((child = glade_xml_search_child (node, GLADE_TAG_PARAMETERS)) != NULL)
 		class->parameters = glade_parameter_list_new_from_node (class->parameters, child);
-	glade_parameter_get_boolean (class->parameters, GLADE_TAG_OPTIONAL, &class->optional);
 		
 	/* Whether or not the property is translatable. This is only used for
 	 * string properties.
 	 */
-	class->translatable = glade_xml_get_property_boolean (node, GLADE_TAG_TRANSLATABLE, TRUE);
+	class->translatable = glade_xml_get_property_boolean (node, GLADE_TAG_TRANSLATABLE, 
+							      class->translatable);
 
 	/* common, optional, etc */
 	class->common   = glade_xml_get_property_boolean (node, GLADE_TAG_COMMON,   class->common);
@@ -1168,6 +1488,13 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 	class->ignore   = glade_xml_get_property_boolean (node, GLADE_TAG_IGNORE,   class->ignore);
 	class->resource = glade_xml_get_property_boolean (node, GLADE_TAG_RESOURCE, class->resource);
 
+	/* No atk introspection here.
+	 */
+	if (glade_xml_get_property_boolean (node, GLADE_TAG_ATK_ACTION, FALSE))
+		class->atk_type = GPC_ATK_ACTION;
+	else if (glade_xml_get_property_boolean (node, GLADE_TAG_ATK_PROPERTY, FALSE))
+		class->atk_type = GPC_ATK_PROPERTY;
+
 	/* Special case pixbuf here.
 	 */
 	if (class->pspec->value_type == GDK_TYPE_PIXBUF)
@@ -1175,82 +1502,14 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 	
 	if (class->optional)
 		class->optional_default =
-			glade_xml_get_property_boolean (node, GLADE_TAG_OPTIONAL_DEFAULT, FALSE);
+			glade_xml_get_property_boolean (node, GLADE_TAG_OPTIONAL_DEFAULT, 
+							class->optional_default);
 
-	/* If this property can't be set with g_object_set, get the work around
-	 * function
+	/* Get any delagate functions for accessing this property
 	 */
-	/* I use here a g_warning to signal these errors instead of a dialog 
-         * box, as if there is one of this kind of errors, there will probably 
-         * a lot of them, and we don't want to inflict the user the pain of 
-         * plenty of dialog boxes.  Ideally, we should collect these errors, 
-         * and show all of them at the end of the load process.
-         */
-	child = glade_xml_search_child (node, GLADE_TAG_SET_FUNCTION);
-	if (child)
-	{
-		gchar *symbol_name = glade_xml_get_content (child);
-
-		if (!module)
-			g_warning (_("The property [%s] of the widget's class [%s] "
-				     "needs a special \"set\" function, but there is "
-				     "no library associated to this widget's class."),
-				   class->name, g_type_name (object_type));
-
-		if (!g_module_symbol (module, symbol_name, (gpointer *)
-				      &class->set_function))
-			g_warning (_("Unable to get the \"set\" function [%s] of the "
-				     "property [%s] of the widget's class [%s] from "
-				     "the module [%s]: %s"),
-				   symbol_name, class->name, g_type_name (object_type),
-				   g_module_name (module), g_module_error ());
-		g_free (symbol_name);
-	}
-
-	/* If this property can't be get with g_object_get, get the work around
-	 * function
-	 */
-	child = glade_xml_search_child (node, GLADE_TAG_GET_FUNCTION);
-	if (child)
-	{
-		gchar *symbol_name = glade_xml_get_content (child);
-
-		if (!module)
-			g_warning (_("The property [%s] of the widget's class [%s] needs a "
-				     "special \"get\" function, but there is no library "
-				     "associated to this widget's class."),
-				   class->name, g_type_name (object_type));
-
-		if (!g_module_symbol(module, symbol_name,
-				     (gpointer *) &class->get_function))
-			g_warning (_("Unable to get the \"get\" function [%s] of the "
-				     "property [%s] of the widget's class [%s] from the "
-				     "module [%s]: %s"),
-				   symbol_name, class->name, g_type_name (object_type),
-				   g_module_name (module), g_module_error ());
-		g_free (symbol_name);
-	}
-
-	child = glade_xml_search_child (node, GLADE_TAG_VERIFY_FUNCTION);
-	if (child)
-	{
-		gchar *symbol_name = glade_xml_get_content (child);
-
-		if (!module)
-			g_warning (_("The property [%s] of the widget's class [%s] needs a "
-				     "special \"get\" function, but there is no library "
-				     "associated to this widget's class."),
-				   class->name, g_type_name (object_type));
-
-		if (!g_module_symbol(module, symbol_name,
-				     (gpointer *) &class->verify_function))
-			g_warning (_("Unable to get the \"verify\" function [%s] of the "
-				     "property [%s] of the widget's class [%s] from the "
-				     "module [%s]: %s"),
-				   symbol_name, class->name, g_type_name (object_type),
-				   g_module_name (module), g_module_error ());
-		g_free (symbol_name);
-	}
+	glade_xml_load_sym_from_node (node, module, GLADE_TAG_SET_FUNCTION,    (gpointer *)&class->set_function);
+	glade_xml_load_sym_from_node (node, module, GLADE_TAG_GET_FUNCTION,    (gpointer *)&class->get_function);
+	glade_xml_load_sym_from_node (node, module, GLADE_TAG_VERIFY_FUNCTION, (gpointer *)&class->verify_function);
 
 	/* notify that we changed the property class */
 	class->is_modified = TRUE;
@@ -1262,7 +1521,6 @@ glade_property_class_update_from_node (GladeXmlNode        *node,
 
 /**
  * glade_property_class_match:
- *
  * @class: a #GladePropertyClass
  * @comp: a #GladePropertyClass
  *
@@ -1280,3 +1538,26 @@ glade_property_class_match (GladePropertyClass *class,
 	return (strcmp (class->id, comp->id) == 0 &&
 		class->pspec->owner_type == comp->pspec->owner_type);
 }
+
+
+/**
+ * glade_property_class_void_value:
+ * @class: a #GladePropertyClass
+ *
+ * Returns: Whether @value for this @class is voided; a voided value
+ *          can be a %NULL value for boxed or object type param specs.
+ */
+gboolean
+glade_property_class_void_value (GladePropertyClass *class,
+				 GValue             *value)
+{
+	g_return_val_if_fail (GLADE_IS_PROPERTY_CLASS (class), FALSE);
+	
+	if (G_IS_PARAM_SPEC_OBJECT (class->pspec) &&
+	    g_value_get_object (value) == NULL)
+		return TRUE;
+	else if (G_IS_PARAM_SPEC_BOXED (class->pspec) &&
+		 g_value_get_boxed (value) == NULL)
+		return TRUE;
+	return FALSE;
+}
