@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * Authors:
  *   Chema Celorio <chema@celorio.com>
@@ -28,6 +28,9 @@
 #include <gdk/gdkkeysyms.h>
 #include <gmodule.h>
 #include <glib/gi18n-lib.h>
+#include <glib/gstdio.h>
+#include <errno.h>
+
 #include "glade.h"
 #include "glade-project.h"
 #include "glade-project-window.h"
@@ -41,6 +44,7 @@
 #include "glade-clipboard.h"
 
 #define GLADE_UTIL_SELECTION_NODE_SIZE 7
+#define GLADE_UTIL_COPY_BUFFSIZE       1024
 
 /* List of widgets that have selection
  */
@@ -59,6 +63,8 @@ glade_util_widget_set_tooltip (GtkWidget *widget, const gchar *str)
 {
 	GtkTooltips *tooltips;
 
+	if (GTK_WIDGET_NO_WINDOW (widget)) return;
+	
 	tooltips = gtk_tooltips_new ();
 	g_object_ref (G_OBJECT (tooltips));
 	gtk_object_sink (GTK_OBJECT (tooltips));
@@ -154,11 +160,10 @@ glade_util_get_type_from_name (const gchar *name)
 
 /**
  * glade_utils_get_pspec_from_funcname:
- * @name:
+ * @funcname: the symbol name of a function to generate a #GParamSpec
  *
- * TODO: write me
- *
- * Returns:
+ * Returns: A #GParamSpec created by the delagate function
+ *          specified by @funcname
  */
 GParamSpec *
 glade_utils_get_pspec_from_funcname (const gchar *funcname)
@@ -184,28 +189,88 @@ glade_utils_get_pspec_from_funcname (const gchar *funcname)
 }
 
 /**
- * glade_util_ui_warn:
+ * glade_util_ui_message:
  * @parent: a #GtkWindow cast as a #GtkWidget
- * @warning: a string
+ * @format: a printf style format string
+ * @type:   a #GladeUIMessageType
+ * @...:    args for the format.
  *
  * Creates a new warning dialog window as a child of @parent containing
- * the text of @warning, runs it, then destroys it on close.
+ * the text of @format, runs it, then destroys it on close. Depending
+ * on @type, a cancel button may apear or the icon may change.
+ *
+ * Returns: True if the @type was GLADE_UI_ARE_YOU_SURE and the user
+ *          selected "OK", True if the @type was GLADE_UI_YES_OR_NO and
+ *          the user selected "YES"; False otherwise.
  */
-void
-glade_util_ui_warn (GtkWidget *parent, const gchar *warning)
+gboolean
+glade_util_ui_message (GtkWidget           *parent, 
+		       GladeUIMessageType   type,
+		       const gchar         *format,
+		       ...)
 {
-	GtkWidget *dialog;
+	GtkWidget      *dialog;
+	GtkMessageType  message_type = GTK_MESSAGE_INFO;
+	GtkButtonsType  buttons_type = GTK_BUTTONS_OK;
+	va_list         args;
+	gchar          *string;
+	gint            response;
+
+	va_start (args, format);
+	string = g_strdup_vprintf (format, args);
+	va_end (args);
+
+	/* Get message_type */
+	switch (type)
+	{
+	case GLADE_UI_INFO: message_type = GTK_MESSAGE_INFO; break;
+	case GLADE_UI_WARN:
+	case GLADE_UI_ARE_YOU_SURE:
+		message_type = GTK_MESSAGE_WARNING;
+		break;
+	case GLADE_UI_ERROR:      message_type = GTK_MESSAGE_ERROR;    break;
+	case GLADE_UI_YES_OR_NO:  message_type = GTK_MESSAGE_QUESTION; break;
+		break;
+	default: 
+		g_critical ("Bad arg for glade_util_ui_message");
+		break;
+	}
+
+
+	/* Get buttons_type */
+	switch (type)
+	{
+	case GLADE_UI_INFO:
+	case GLADE_UI_WARN:
+	case GLADE_UI_ERROR:
+		buttons_type = GTK_BUTTONS_OK;
+		break;
+	case GLADE_UI_ARE_YOU_SURE: buttons_type = GTK_BUTTONS_OK_CANCEL; break;
+	case GLADE_UI_YES_OR_NO:    buttons_type = GTK_BUTTONS_YES_NO; break;
+		break;
+	default: 
+		g_critical ("Bad arg for glade_util_ui_message");
+		break;
+	}
 
 	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
-					 GTK_MESSAGE_WARNING,
-					 GTK_BUTTONS_OK,
-					 warning);
+					 message_type,
+					 buttons_type,
+					 string);
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
-	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
+	g_free (string);
+
+	return (response == GTK_RESPONSE_OK || response == GTK_RESPONSE_YES);
 }
 
+
+
+
+
 typedef struct {
 	GtkStatusbar *statusbar;
 	guint context_id;
@@ -430,7 +495,7 @@ glade_util_hide_window (GtkWindow *window)
  * @parent: the parent #GtkWindow for the dialog
  * @action: a #GladeUtilFileDialogType to say if the dialog will open or save
  *
- * Returns: a file chooser dialog. The caller is responsible 
+ * Returns: a "glade file" file chooser dialog. The caller is responsible 
  *          for showing the dialog
  */
 GtkWidget *
@@ -438,17 +503,34 @@ glade_util_file_dialog_new (const gchar *title, GtkWindow *parent,
 			     GladeUtilFileDialogType action)
 {
 	GtkWidget *file_dialog;
+	GtkFileFilter *file_filter;
 
 	g_return_val_if_fail ((action == GLADE_FILE_DIALOG_ACTION_OPEN ||
 			       action == GLADE_FILE_DIALOG_ACTION_SAVE), NULL);
-
+	
 	file_dialog = gtk_file_chooser_dialog_new (title, parent, action,
-						    GTK_STOCK_CANCEL,
-						    GTK_RESPONSE_CANCEL,
-						    action == GLADE_FILE_DIALOG_ACTION_OPEN ?
-						    GTK_STOCK_OPEN : GTK_STOCK_SAVE,
-						    GTK_RESPONSE_OK,
-						    NULL);
+						   GTK_STOCK_CANCEL,
+						   GTK_RESPONSE_CANCEL,
+						   action == GLADE_FILE_DIALOG_ACTION_OPEN ?
+						   GTK_STOCK_OPEN : GTK_STOCK_SAVE,
+						   GTK_RESPONSE_OK,
+						   NULL);
+	
+	file_filter = gtk_file_filter_new ();
+	gtk_file_filter_add_pattern (file_filter, "*");
+	gtk_file_filter_set_name (file_filter, _("All Files"));
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_dialog), file_filter);
+
+	file_filter = gtk_file_filter_new ();
+	gtk_file_filter_add_pattern (file_filter, "*.glade");
+	gtk_file_filter_set_name (file_filter, _("Glade Files"));
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_dialog), file_filter);
+
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_dialog), file_filter);
+
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (file_dialog), TRUE);
+	gtk_dialog_set_default_response (GTK_DIALOG (file_dialog), GTK_RESPONSE_OK);
+	
 	gtk_window_set_position (GTK_WINDOW (file_dialog), GTK_WIN_POS_CENTER);
 
 	return file_dialog;
@@ -476,6 +558,33 @@ glade_util_replace (gchar *str, gchar a, gchar b)
 	}
 }
 
+/**
+ * glade_util_read_prop_name:
+ * @str: a string
+ *
+ * Return a usable version of a property identifier as found
+ * in a freshly parserd #GladeInterface
+ */
+gchar *
+glade_util_read_prop_name (const gchar *str)
+{
+	gchar *id = g_strdup (str);
+
+	glade_util_replace (id, '_', '-');
+
+	if (strstr (id, "::"))
+	{
+		/* Extract the second half of "AtkObject::accessible_name"
+		 */
+		gchar **split = g_strsplit (id, "::", 0);
+		g_free (id);
+		id = g_strdup (split[1]);
+		g_strfreev (split);
+	}
+	return id;
+}
+
+
 /**
  * glade_util_duplicate_underscores:
  * @name: a string
@@ -485,13 +594,13 @@ glade_util_replace (gchar *str, gchar a, gchar b)
  *
  * Returns: a newly allocated string
  */
-char *
-glade_util_duplicate_underscores (const char *name)
+gchar *
+glade_util_duplicate_underscores (const gchar *name)
 {
-	const char *tmp;
-	const char *last_tmp = name;
-	char *underscored_name = g_malloc (strlen (name) * 2 + 1);
-	char *tmp_underscored = underscored_name;
+	const gchar *tmp;
+	const gchar *last_tmp = name;
+	gchar *underscored_name = g_malloc (strlen (name) * 2 + 1);
+	gchar *tmp_underscored = underscored_name;
 
 	for (tmp = last_tmp; *tmp; tmp = g_utf8_next_char (tmp))
 	{
@@ -661,7 +770,7 @@ glade_util_draw_nodes_idle (GdkWindow *expose_win)
 		goto out;
 
 	/* Find the corresponding GtkWidget */
-	gdk_window_get_user_data (expose_win, (gpointer *)&expose_widget);
+	gdk_window_get_user_data (expose_win, (gpointer)&expose_widget);
 
 	gc = expose_widget->style->black_gc;
 
@@ -806,7 +915,7 @@ glade_util_clear_selection (void)
 }
 
 /**
- * glade_util_has_selectoin:
+ * glade_util_has_selection:
  * @widget: a #GtkWidget
  *
  * Returns: %TRUE if @widget has visual selection, %FALSE otherwise
@@ -829,6 +938,22 @@ glade_util_get_selection ()
 	return glade_util_selection;
 }
 
+/**
+ * glade_util_selected_placeholder:
+ *
+ * Returns: The selected placeholder, if there
+ * is only one selected item and that item is a placeholder.
+ */
+GladePlaceholder *
+glade_util_selected_placeholder ()
+{
+	if (g_list_length (glade_util_selection) == 1 &&
+	    GLADE_IS_PLACEHOLDER (glade_util_selection->data))
+		return glade_util_selection->data;
+
+	return NULL;
+}
+
 /*
  * taken from gtk... maybe someday we can convince them to
  * expose gtk_container_get_all_children
@@ -847,6 +972,13 @@ gtk_container_children_callback (GtkWidget *widget,
  * glade_util_container_get_all_children:
  * @container: a #GtkContainer
  *
+ * Use this to itterate over all children in a GtkContainer,
+ * as it used _forall() instead of _foreach() (and the GTK+ version
+ * of this function is simply not exposed).
+ *
+ * Note that glade_widget_class_get_children() is the high-level
+ * abstraction and will usually end up calling this function.
+ *
  * Returns: a #GList giving the contents of @container
  */
 GList *
@@ -860,6 +992,7 @@ glade_util_container_get_all_children (GtkContainer *container)
 			      gtk_container_children_callback,
 			      &children);
 
+	/* Preserve the natural order by reversing the list */
 	return g_list_reverse (children);
 }
 
@@ -955,11 +1088,10 @@ glade_util_gtkcontainer_relation (GladeWidget *parent, GladeWidget *widget)
 
 /**
  * glade_util_any_gtkcontainer_relation:
- * @widget: a GladeWidget
- * @parent: a GladeWidget
+ * @widgets: a #GList of #GladeWidgets
+ * @parent:  the parent #GladeWidget
  *
- *
- * Returns whether any of these widgets are parented by a GtkContainer
+ * Returns: whether any of these widgets are parented by a GtkContainer
  * through the GtkContainer interface.
  */
 gboolean
@@ -1017,201 +1149,6 @@ glade_util_count_placeholders (GladeWidget *parent)
 	return placeholders;
 }
 
-/**
- * glade_util_paste_clipboard:
- * @placeholder: The #GladePlaceholder to paste the clipboard item to,
- *               if NULL; paste to the selected object in the active project.
- *
- * Paste the clipboard selection to the active project's 
- * selection (the project must have only one object selected).
- */
-void
-glade_util_paste_clipboard (GladePlaceholder *placeholder,
-			    GladeWidget      *parent)
-{
-	GladeClipboard     *clipboard = glade_default_app_get_clipboard ();
-	GList              *list;
-	GladeWidget        *widget;
-	gint                gtkcontainer_relations = 0;
-
-	g_return_if_fail (placeholder == NULL || GLADE_IS_PLACEHOLDER (placeholder));
-	g_return_if_fail (parent      == NULL || GLADE_IS_WIDGET (parent));
-      
-	if ((list = glade_default_app_get_selection ()) != NULL)
-	{
-		if (placeholder == NULL &&
-		    g_list_length (list) != 1)
-		{
-			glade_util_ui_warn 
-				(glade_default_app_get_window(),
-				 _("Unable to paste to multiple widgets"));
-			return;
-		}
-	}
-	
-	if (g_list_length (clipboard->selection) == 0)
-	{
-		glade_util_ui_warn (glade_default_app_get_window (),
-				    _("No widget selected on the clipboard"));
-		return;
-	}
-	
-	for (list = clipboard->selection; 
-	     list && list->data; list = list->next)
-	{
-		widget = list->data;
-
-		if (GTK_WIDGET_TOPLEVEL (widget->object) == FALSE && parent)
-		{
-			/* Ensure a paste is supported
-			 */
-			if (glade_util_widget_pastable (widget, parent) == FALSE)
-			{
-				gchar *message = g_strdup_printf
-					(_("Unable to paste widget %s to parent %s"),
-					 widget->name, parent->name);
-				glade_util_ui_warn (glade_default_app_get_window (), message);
-				g_free (message);
-				return;
-			}
-
-			/* Count gtk container relations
-			 */
-			if (glade_util_gtkcontainer_relation (parent, widget))
-				gtkcontainer_relations++;
-		}
-	}
-
-	g_assert (widget);
-
-	/* Ensure enough placeholders are available */
-	if (GTK_WIDGET_TOPLEVEL (widget->object) == FALSE &&
-	    parent && parent->manager != NULL &&
-	    gtkcontainer_relations != 1) 
-	{
-		glade_util_ui_warn (glade_default_app_get_window (), 
-				    _("Only one widget can be pasted at a "
-				      "time to this container"));
-		return;
-	}
-
-	if (parent && parent->manager == NULL &&
-	    glade_util_count_placeholders (parent) < gtkcontainer_relations)
-	{
-		glade_util_ui_warn (glade_default_app_get_window (), 
-				    _("Insufficient amount of placeholders in "
-				      "target container"));
-		return;
-	}
-
-	glade_command_paste (clipboard->selection, parent, placeholder);
-}
-
-/**
- * glade_util_delete_clipboard:
- *
- * Cut the active project's selection.
- */
-void
-glade_util_delete_clipboard (void)
-{
-	GladeClipboard     *clipboard = glade_default_app_get_clipboard ();
-
-	glade_command_delete (clipboard->selection);
-}
-
-
-/**
- * glade_util_cut_selection:
- *
- * Cut the active project's selection.
- */
-void
-glade_util_cut_selection (void)
-{
-	GList              *widgets = NULL, *list;
-	GladeWidget        *widget;
-	
-	for (list = glade_default_app_get_selection ();
-	     list && list->data; list = list->next)
-	{
-		widget  = glade_widget_get_from_gobject (GTK_WIDGET (list->data));
-		widgets = g_list_prepend (widgets, widget);
-	}
-
-	if (widgets)
-	{
-		glade_command_cut (widgets);
-		g_list_free (widgets);
-	}
-	else
-	{
-		glade_util_ui_warn (glade_default_app_get_window (),
-							_("No widget selected!"));
-	}
-}
-
-/**
- * glade_util_copy_selection:
- *
- * Copy the active project's selection.
- */
-void
-glade_util_copy_selection (void)
-{
-	GList              *widgets = NULL, *list;
-	GladeWidget        *widget;
-	
-	for (list = glade_default_app_get_selection ();
-	     list && list->data; list = list->next)
-	{
-		if ((widget = 
-		     glade_widget_get_from_gobject (list->data)) != NULL)
-			widgets = g_list_prepend (widgets, widget);
-	}
-
-	if (widgets)
-	{
-		glade_command_copy (widgets);
-		g_list_free (widgets);
-	}
-	else
-	{
-		glade_util_ui_warn (glade_default_app_get_window(),
-							_("No widget selected!"));
-	}
-}
-
-/**
- * glade_util_delete_selection:
- *
- * Delete the active project's selection.
- */
-void
-glade_util_delete_selection (void)
-{
-	GList              *widgets = NULL, *list;
-	GladeWidget        *widget;
-
-	for (list = glade_default_app_get_selection ();
-	     list && list->data; list = list->next)
-	{
-		widget  = glade_widget_get_from_gobject (GTK_WIDGET (list->data));
-		widgets = g_list_prepend (widgets, widget);
-	}
-
-	if (widgets)
-	{
-		glade_command_delete (widgets);
-		g_list_free (widgets);
-	}
-	else
-	{
-		glade_util_ui_warn (glade_default_app_get_window(),
-							_("No widget selected!"));
-	}
-}
-
 static GtkTreeIter *
 glade_util_find_iter (GtkTreeModel *model,
 		      GtkTreeIter  *iter,
@@ -1283,7 +1220,7 @@ glade_util_find_iter_by_widget (GtkTreeModel *model,
 	return NULL;
 }
 
-LIBGLADEUI_API gboolean
+gboolean
 glade_util_basenames_match  (const gchar  *path1,
 			     const gchar  *path2)
 {
@@ -1303,3 +1240,368 @@ glade_util_basenames_match  (const gchar  *path1,
 	}
 	return match;
 }
+
+/**
+ * glade_util_purify_list:
+ * @list: A #GList
+ *
+ * Returns: A newly allocated version of @list with no 
+ *          duplicate data entries
+ */
+GList *
+glade_util_purify_list (GList *list)
+{
+	GList *l, *newlist = NULL;
+
+	for (l = list; l; l = l->next)
+		if (!g_list_find (newlist, l->data))
+			newlist = g_list_prepend (newlist, l->data);
+
+	g_list_free (list);
+
+	return g_list_reverse (newlist);
+}
+
+/**
+ * glade_util_added_in_list:
+ * @old: the old #GList
+ * @new: the new #GList
+ *
+ * Returns: A newly allocated #GList of elements that
+ *          are in @new but not in @old
+ *
+ */
+GList *
+glade_util_added_in_list (GList *old,
+			  GList *new)
+{
+	GList *added = NULL, *list;
+
+	for (list = new; list; list = list->next)
+	{
+		if (!g_list_find (old, list->data))
+			added = g_list_prepend (added, list->data);
+	} 
+
+	return g_list_reverse (added);
+}
+
+/**
+ * glade_util_removed_from_list:
+ * @old: the old #GList
+ * @new: the new #GList
+ *
+ * Returns: A newly allocated #GList of elements that
+ *          are in @old no longer in @new
+ *
+ */
+GList *
+glade_util_removed_from_list (GList *old,
+			      GList *new)
+{
+	GList *added = NULL, *list;
+
+	for (list = old; list; list = list->next)
+	{
+		if (!g_list_find (new, list->data))
+			added = g_list_prepend (added, list->data);
+	} 
+
+	return g_list_reverse (added);
+}
+
+
+/**
+ * glade_util_canonical_path:
+ * @path: any path that may contain ".." or "." components
+ *
+ * Returns: an absolute path to the specified file or directory
+ *          that contains no ".." or "." components (this does
+ *          not call readlink like realpath() does).
+ *
+ * Note: on some systems; I think its possible that we dont have
+ *       permission to execute in the directory in which the glade
+ *       file resides; I decided finally to do it this way anyway
+ *       since libc's realpath() does exactly the same.
+ */
+gchar *
+glade_util_canonical_path (const gchar *path)
+{
+	gchar *orig_dir, *dirname, *basename, *direct_dir, *direct_name = NULL;
+
+	g_return_val_if_fail (path != NULL, NULL);
+
+	basename = g_path_get_basename (path);
+
+	if ((orig_dir = g_get_current_dir ()) != NULL)
+	{
+		if ((dirname = g_path_get_dirname (path)) != NULL)
+		{
+			if (g_chdir (dirname) == 0)
+			{
+				if ((direct_dir = g_get_current_dir ()) != NULL)
+					direct_name = 
+						g_build_filename (direct_dir, basename, NULL);
+				else
+					g_warning ("g_path");
+
+				if (g_chdir (orig_dir) != 0)
+					g_warning ("Unable to chdir back to %s directory (%s)",
+						   orig_dir, g_strerror (errno));
+
+			} else g_warning ("Unable to chdir to %s directory (%s)", 
+					  dirname, g_strerror (errno));
+
+			g_free (dirname);
+		} else g_warning ("Unable to get directory component of %s\n", path);
+		g_free (orig_dir);
+	}
+
+	if (basename) g_free (basename);
+
+	return direct_name;
+}
+
+static gboolean
+glade_util_canonical_match (const gchar  *src_path,
+			    const gchar  *dest_path)
+{
+	gchar      *canonical_src, *canonical_dest;
+	gboolean    match;
+	canonical_src  = glade_util_canonical_path (src_path);
+	canonical_dest = glade_util_canonical_path (dest_path);
+
+	match = (strcmp (canonical_src, canonical_dest) == 0);
+
+	g_free (canonical_src);
+	g_free (canonical_dest);
+	
+	return match;
+}
+
+/**
+ * glade_util_copy_file:
+ * @src_path:  the path to the source file
+ * @dest_path: the path to the destination file to create or overwrite.
+ *
+ * Copies a file from @src to @dest, queries the user
+ * if it involves overwriting the target and displays an
+ * error message upon failure.
+ *
+ * Returns: True if the copy was successfull.
+ */
+gboolean
+glade_util_copy_file (const gchar  *src_path,
+		      const gchar  *dest_path)
+{
+	GIOChannel *src, *dest;
+	GError     *error = NULL;
+	GIOStatus   read_status, write_status = G_IO_STATUS_ERROR;
+	gchar       buffer[GLADE_UTIL_COPY_BUFFSIZE];
+	gsize       bytes_read, bytes_written, written;
+	gboolean    success = FALSE;
+
+	/* FIXME: This may break if src_path & dest_path are actually 
+	 * the same file, right now the canonical comparison is the
+	 * best check I have.
+	 */
+	if (glade_util_canonical_match (src_path, dest_path))
+		return FALSE;
+
+	if (g_file_test (dest_path, G_FILE_TEST_IS_REGULAR) == TRUE)
+		if (glade_util_ui_message
+		    (glade_app_get_window(), GLADE_UI_YES_OR_NO,
+		     _("%s exists.\nDo you want to replace it?"), dest_path) == FALSE)
+		    return FALSE;
+
+
+	if ((src = g_io_channel_new_file (src_path, "r", &error)) != NULL)
+	{
+		g_io_channel_set_encoding (src, NULL, NULL);
+
+		if ((dest = g_io_channel_new_file (dest_path, "w", &error)) != NULL)
+		{
+			g_io_channel_set_encoding (dest, NULL, NULL);
+
+			while ((read_status = g_io_channel_read_chars 
+				(src, buffer, GLADE_UTIL_COPY_BUFFSIZE,
+				 &bytes_read, &error)) != G_IO_STATUS_ERROR)
+			{
+				bytes_written = 0;
+				while ((write_status = g_io_channel_write_chars 
+					(dest, buffer + bytes_written, 
+					 bytes_read - bytes_written, 
+					 &written, &error)) != G_IO_STATUS_ERROR &&
+				       (bytes_written + written) < bytes_read)
+					bytes_written += written;
+				
+				if (write_status == G_IO_STATUS_ERROR)
+				{
+					glade_util_ui_message (glade_app_get_window(),
+							       GLADE_UI_ERROR,
+							       _("Error writing to %s: %s"),
+							       dest_path, error->message);
+					error = (g_error_free (error), NULL);
+					break;
+				}
+
+				/* Break on EOF & ERROR but not AGAIN and not NORMAL */
+				if (read_status == G_IO_STATUS_EOF) break;
+			}
+			
+			if (read_status == G_IO_STATUS_ERROR)
+			{
+				glade_util_ui_message (glade_app_get_window(),
+						       GLADE_UI_ERROR,
+						       _("Error reading %s: %s"),
+						       src_path, error->message);
+				error = (g_error_free (error), NULL);
+			}
+
+
+			/* From here on, unless we have problems shutting down, succuss ! */
+			success = (read_status == G_IO_STATUS_EOF && 
+				   write_status == G_IO_STATUS_NORMAL);
+
+			if (g_io_channel_shutdown (dest, TRUE, &error) != G_IO_STATUS_NORMAL)
+			{
+				glade_util_ui_message
+					(glade_app_get_window(),
+					 GLADE_UI_ERROR,
+					 _("Error shutting down I/O channel %s: %s"),
+						       dest_path, error->message);
+				error = (g_error_free (error), NULL);
+				success = FALSE;
+			}
+		}
+		else
+		{
+			glade_util_ui_message (glade_app_get_window(),
+					       GLADE_UI_ERROR,
+					       _("Failed to open %s for writing: %s"), 
+					       dest_path, error->message);
+			error = (g_error_free (error), NULL);
+
+		}
+
+
+		if (g_io_channel_shutdown (src, TRUE, &error) != G_IO_STATUS_NORMAL)
+		{
+			glade_util_ui_message (glade_app_get_window(),
+					       GLADE_UI_ERROR,
+					       _("Error shutting down io channel %s: %s"),
+					       src_path, error->message);
+			success = FALSE;
+		}
+	}
+	else 
+	{
+		glade_util_ui_message (glade_app_get_window(),
+				       GLADE_UI_ERROR,
+				       _("Failed to open %s for reading: %s"), 
+				       src_path, error->message);
+		error = (g_error_free (error), NULL);
+	}
+	return success;
+}
+
+/**
+ * glade_util_class_implements_interface:
+ * @class_type: A #GType
+ * @iface_type: A #GType
+ *
+ * Returns: whether @class_type implements the @iface_type interface
+ */
+gboolean
+glade_util_class_implements_interface (GType class_type, 
+				       GType iface_type)
+{
+	GType    *ifaces;
+	guint     n_ifaces, i;
+	gboolean  implemented = FALSE;
+
+	if ((ifaces = g_type_interfaces (class_type, &n_ifaces)) != NULL)
+	{
+		for (i = 0; i < n_ifaces; i++)
+			if (ifaces[i] == iface_type)
+			{
+				implemented = TRUE;
+				break;
+			}
+		g_free (ifaces);
+	}
+	return implemented;
+}
+
+
+/**
+ * glade_util_load_library:
+ * @library_name: name of the library
+ *
+ * Loads the named library from the Glade modules directory, or failing that
+ * from the standard platform specific directories.
+ *
+ * The @library_name should not include any platform specifix prefix or suffix,
+ * those are automatically added, if needed, by g_module_build_path()
+ *
+ * Returns: a #GModule on success, or %NULL on failure.
+ */
+GModule *
+glade_util_load_library (const gchar *library_name)
+{
+	gchar   *path;
+	GModule *module;
+
+	path = g_module_build_path (glade_modules_dir, library_name);
+	if (!path)
+	{
+		g_warning (_("Not enough memory."));
+		return NULL;
+	}
+
+	module = g_module_open (path, G_MODULE_BIND_LAZY);
+	g_free (path);
+
+	if (!module)
+	{
+		path = g_module_build_path (NULL, library_name);
+		if (!path)
+		{
+			g_warning (_("Not enough memory."));
+			return NULL;
+		}
+
+		module = g_module_open (path, G_MODULE_BIND_LAZY);
+		g_free (path);
+	}
+
+	if (!module)
+	{
+		g_warning (_("Unable to open the module %s."), library_name);
+	}
+
+	return module;
+}
+
+/**
+ * glade_util_file_is_writeable:
+ * @path:  the path to the file
+ *
+ * Checks whether the file at @path is writeable
+ *
+ * Returns: TRUE if file is writeable
+ */
+gboolean
+glade_util_file_is_writeable (const gchar *path)
+{
+	GIOChannel *channel;
+	g_return_val_if_fail (path != NULL, FALSE);
+
+	/* The only way to really know if the file is writable */
+	if ((channel = g_io_channel_new_file (path, "a+", NULL)) != NULL)
+	{
+		g_io_channel_unref (channel);
+		return TRUE;
+	}
+	return FALSE;
+}
